@@ -288,15 +288,6 @@ static void init_audio_filtergraph(mlt_filter filter,
         mlt_log_error(filter, "Cannot set sink sample rates\n");
         goto fail;
     }
-    ret = av_opt_set_int_list(pdata->avbuffsink_ctx,
-                              "channel_counts",
-                              channel_counts,
-                              -1,
-                              AV_OPT_SEARCH_CHILDREN);
-    if (ret < 0) {
-        mlt_log_error(filter, "Cannot set sink channel counts\n");
-        goto fail;
-    }
 #if HAVE_FFMPEG_CH_LAYOUT
     ret = av_opt_set(pdata->avbuffsink_ctx,
                      "ch_layouts",
@@ -307,6 +298,15 @@ static void init_audio_filtergraph(mlt_filter filter,
         goto fail;
     }
 #else
+    ret = av_opt_set_int_list(pdata->avbuffsink_ctx,
+                              "channel_counts",
+                              channel_counts,
+                              -1,
+                              AV_OPT_SEARCH_CHILDREN);
+    if (ret < 0) {
+        mlt_log_error(filter, "Cannot set sink channel counts\n");
+        goto fail;
+    }
     ret = av_opt_set_int_list(pdata->avbuffsink_ctx,
                               "channel_layouts",
                               channel_layouts,
@@ -777,7 +777,8 @@ static int filter_get_image(mlt_frame frame,
 
     mlt_log_debug(MLT_FILTER_SERVICE(filter), "position %" PRId64 "\n", pos);
     if (mlt_properties_get_int(MLT_FILTER_PROPERTIES(filter), "_yuv_only")) {
-        *format = mlt_image_yuv422;
+        if (*format == mlt_image_rgb || *format == mlt_image_rgba)
+            *format = mlt_image_yuv422;
     } else {
         *format = mlt_get_supported_image_format(*format);
     }
@@ -808,26 +809,32 @@ static int filter_get_image(mlt_frame frame,
         pdata->avinframe->color_primaries = mlt_properties_get_int(frame_properties,
                                                                    "color_primaries");
         pdata->avinframe->color_trc = mlt_properties_get_int(frame_properties, "color_trc");
-        pdata->avinframe->color_range = mlt_properties_get_int(frame_properties, "full_range")
-                                            ? AVCOL_RANGE_JPEG
-                                            : AVCOL_RANGE_MPEG;
+        // Setting full_range here causes full range input to always be down-converted to limited
+        // range when operating in YUV, regardless of the consumer.color_range -- for each avfilter!
+        // pdata->avinframe->color_range = mlt_properties_get_int(frame_properties, "full_range")
+        //                                     ? AVCOL_RANGE_JPEG
+        //                                     : AVCOL_RANGE_MPEG;
 
-        switch (mlt_properties_get_int(frame_properties, "colorspace")) {
-        case 240:
-            pdata->avinframe->colorspace = AVCOL_SPC_SMPTE240M;
-            break;
-        case 601:
-            pdata->avinframe->colorspace = AVCOL_SPC_BT470BG;
-            break;
-        case 709:
-            pdata->avinframe->colorspace = AVCOL_SPC_BT709;
-            break;
-        case 2020:
-            pdata->avinframe->colorspace = AVCOL_SPC_BT2020_NCL;
-            break;
-        case 2021:
-            pdata->avinframe->colorspace = AVCOL_SPC_BT2020_CL;
-            break;
+        if (*format == mlt_image_rgb || *format == mlt_image_rgba) {
+            pdata->avinframe->colorspace = AVCOL_SPC_RGB;
+        } else {
+            switch (mlt_properties_get_int(frame_properties, "colorspace")) {
+            case 240:
+                pdata->avinframe->colorspace = AVCOL_SPC_SMPTE240M;
+                break;
+            case 601:
+                pdata->avinframe->colorspace = AVCOL_SPC_BT470BG;
+                break;
+            case 709:
+                pdata->avinframe->colorspace = AVCOL_SPC_BT709;
+                break;
+            case 2020:
+                pdata->avinframe->colorspace = AVCOL_SPC_BT2020_NCL;
+                break;
+            case 2021:
+                pdata->avinframe->colorspace = AVCOL_SPC_BT2020_CL;
+                break;
+            }
         }
 
         ret = av_frame_get_buffer(pdata->avinframe, 1);
@@ -836,18 +843,24 @@ static int filter_get_image(mlt_frame frame,
         }
 
         // Set up the input frame
-        if (*format == mlt_image_yuv420p) {
+        if (*format == mlt_image_yuv420p || *format == mlt_image_yuv420p10
+            || *format == mlt_image_yuv444p10 || *format == mlt_image_yuv422p16) {
             int i = 0;
             int p = 0;
-            int widths[3] = {*width, *width / 2, *width / 2};
-            int heights[3] = {*height, *height / 2, *height / 2};
-            uint8_t *src = *image;
+            int strides[4];
+            uint8_t *planes[4];
+            int heights[3] = {*height, *height, *height};
+            if (*format == mlt_image_yuv420p || *format == mlt_image_yuv420p10) {
+                heights[1] >>= 1;
+                heights[2] >>= 1;
+            }
+            mlt_image_format_planes(*format, *width, *height, *image, planes, strides);
             for (p = 0; p < 3; p++) {
-                uint8_t *dst = pdata->avinframe->data[p];
+                uint8_t *const dst = pdata->avinframe->data[p];
                 for (i = 0; i < heights[p]; i++) {
-                    memcpy(dst, src, widths[p]);
-                    src += widths[p];
-                    dst += pdata->avinframe->linesize[p];
+                    memcpy(&dst[i * pdata->avinframe->linesize[p]],
+                           &planes[p][i * strides[p]],
+                           strides[p]);
                 }
             }
         } else {
@@ -880,18 +893,24 @@ static int filter_get_image(mlt_frame frame,
         }
 
         // Copy the filter output into the original buffer
-        if (*format == mlt_image_yuv420p) {
+        if (*format == mlt_image_yuv420p || *format == mlt_image_yuv420p10
+            || *format == mlt_image_yuv444p10 || *format == mlt_image_yuv422p16) {
             int i = 0;
             int p = 0;
-            int widths[3] = {*width, *width / 2, *width / 2};
-            int heights[3] = {*height, *height / 2, *height / 2};
-            uint8_t *dst = *image;
+            int strides[4];
+            uint8_t *planes[4];
+            int heights[3] = {*height, *height, *height};
+            if (*format == mlt_image_yuv420p || *format == mlt_image_yuv420p10) {
+                heights[1] >>= 1;
+                heights[2] >>= 1;
+            }
+            mlt_image_format_planes(*format, *width, *height, *image, planes, strides);
             for (p = 0; p < 3; p++) {
                 uint8_t *src = pdata->avoutframe->data[p];
                 for (i = 0; i < heights[p]; i++) {
-                    memcpy(dst, src, widths[p]);
-                    dst += widths[p];
-                    src += pdata->avoutframe->linesize[p];
+                    memcpy(&planes[p][i * strides[p]],
+                           &src[i * pdata->avoutframe->linesize[p]],
+                           strides[p]);
                 }
             }
         } else {
