@@ -1,6 +1,6 @@
 /*
  * consumer_decklink.cpp -- output through Blackmagic Design DeckLink
- * Copyright (C) 2010-2023 Meltytech, LLC
+ * Copyright (C) 2010-2025 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -49,6 +49,7 @@ static int swab_sliced(int id, int idx, int jobs, void *cookie)
 static const unsigned PREROLL_MINIMUM = 3;
 
 enum { OP_NONE = 0, OP_OPEN, OP_START, OP_STOP, OP_EXIT };
+enum { EOTF_SDR = 0, EOTF_HDR = 1, EOTF_PQ = 2, EOTF_HLG = 3 }; ///< CEA 861.3
 
 class DeckLinkConsumer : public IDeckLinkVideoOutputCallback, public IDeckLinkAudioOutputCallback
 {
@@ -65,6 +66,7 @@ private:
     uint64_t m_count;
     int m_outChannels;
     int m_inChannels;
+    bool m_fix5p1{false};
     bool m_isAudio;
     int m_isKeyer;
     IDeckLinkKeyer *m_deckLinkKeyer;
@@ -311,8 +313,8 @@ protected:
         }
 
         // Get the keyer interface
-        IDeckLinkAttributes *deckLinkAttributes = 0;
-        if (m_deckLink->QueryInterface(IID_IDeckLinkAttributes, (void **) &deckLinkAttributes)
+        IDeckLinkProfileAttributes *deckLinkAttributes = 0;
+        if (m_deckLink->QueryInterface(IID_IDeckLinkProfileAttributes, (void **) &deckLinkAttributes)
             == S_OK) {
 #ifdef _WIN32
             BOOL flag = FALSE;
@@ -348,7 +350,7 @@ protected:
         if (!mlt_properties_get_int(properties, "running"))
             return 0;
 
-        mlt_log_verbose(getConsumer(), "preroll %u frames\n", m_preroll);
+        mlt_log_debug(getConsumer(), "preroll %u frames\n", m_preroll);
 
         // preroll frames
         for (unsigned i = 0; i < m_preroll; i++)
@@ -381,6 +383,10 @@ protected:
         } else {
             m_outChannels = 16;
         }
+        auto channelLayout = mlt_audio_channel_layout_id(
+            mlt_properties_get(properties, "channel_layout"));
+        m_fix5p1 = (channelLayout == mlt_channel_auto && m_inChannels == 6)
+                   || channelLayout == mlt_channel_5p1 || channelLayout == mlt_channel_5p1_back;
         m_isAudio = !mlt_properties_get_int(properties, "audio_off");
         m_terminate_on_pause = mlt_properties_get_int(properties, "terminate_on_pause");
 
@@ -524,8 +530,18 @@ protected:
         m_sliced_swab = mlt_properties_get_int(consumer_properties, "sliced_swab");
 
         if (rendered && !mlt_frame_get_image(frame, &image, &format, &m_width, &height, 0)) {
-            if (decklinkFrame)
-                decklinkFrame->GetBytes((void **) &m_buffer);
+            if (decklinkFrame) {
+                IDeckLinkVideoBuffer *videoBuffer = NULL;
+                if (decklinkFrame->QueryInterface(IID_IDeckLinkVideoBuffer, (void **) &videoBuffer)
+                    == S_OK) {
+                    if (videoBuffer->StartAccess(bmdBufferAccessWrite) == S_OK) {
+                        videoBuffer->GetBytes((void **) &m_buffer);
+                        videoBuffer->EndAccess(bmdBufferAccessWrite);
+                    }
+                    videoBuffer->Release();
+                    videoBuffer = NULL;
+                }
+            }
 
             if (m_buffer) {
                 // NTSC SDI is always 486 lines
@@ -572,9 +588,18 @@ protected:
             }
         } else if (decklinkFrame) {
             uint8_t *buffer = NULL;
-            decklinkFrame->GetBytes((void **) &buffer);
-            if (buffer)
-                memcpy(buffer, m_buffer, stride * height);
+            IDeckLinkVideoBuffer *videoBuffer = NULL;
+            if (decklinkFrame->QueryInterface(IID_IDeckLinkVideoBuffer, (void **) &videoBuffer)
+                == S_OK) {
+                if (videoBuffer->StartAccess(bmdBufferAccessWrite) == S_OK) {
+                    videoBuffer->GetBytes((void **) &buffer);
+                    if (buffer)
+                        memcpy(buffer, m_buffer, stride * height);
+                    videoBuffer->EndAccess(bmdBufferAccessWrite);
+                }
+                videoBuffer->Release();
+                videoBuffer = NULL;
+            }
         }
         if (decklinkFrame) {
             char *vitc;
@@ -600,6 +625,78 @@ protected:
                                           mlt_properties_get_int(MLT_FRAME_PROPERTIES(frame),
                                                                  "meta.attr.vitc.userbits"));
 
+            // Set color space
+            IDeckLinkVideoFrameMutableMetadataExtensions *frameMeta = nullptr;
+            if (decklinkFrame->QueryInterface(IID_IDeckLinkVideoFrameMutableMetadataExtensions,
+                                              (void **) &frameMeta)
+                == S_OK) {
+                auto colorspace = bmdColorspaceRec709;
+                switch (mlt_properties_get_int(consumer_properties, "colorspace")) {
+                case 601:
+                    colorspace = bmdColorspaceRec601;
+                    break;
+                case 2020:
+                    colorspace = bmdColorspaceRec2020;
+                    break;
+                }
+                frameMeta->SetInt(bmdDeckLinkFrameMetadataColorspace, colorspace);
+
+                // Set HDR a transfer function
+                if (mlt_properties_exists(consumer_properties, "color_trc")) {
+                    if (!strcmp("arib-std-b67",
+                                mlt_properties_get(consumer_properties, "color_trc"))) {
+                        frameMeta->SetInt(bmdDeckLinkFrameMetadataHDRElectroOpticalTransferFunc,
+                                          EOTF_HLG);
+                        decklinkFrame->SetFlags(decklinkFrame->GetFlags()
+                                                & ~bmdFrameContainsHDRMetadata);
+                    } else if (!strcmp("smpte2084",
+                                       mlt_properties_get(consumer_properties, "color_trc"))) {
+                        frameMeta->SetInt(bmdDeckLinkFrameMetadataHDRElectroOpticalTransferFunc,
+                                          EOTF_PQ);
+                        // CEA/SMPTE HDR metadata
+                        // TODO: document and provide defaults for these
+                        decklinkFrame->SetFlags(decklinkFrame->GetFlags()
+                                                | bmdFrameContainsHDRMetadata);
+                        frameMeta->SetFloat(bmdDeckLinkFrameMetadataHDRDisplayPrimariesRedX,
+                                            mlt_properties_get_double(consumer_properties,
+                                                                      "hdr_red_x"));
+                        frameMeta->SetFloat(bmdDeckLinkFrameMetadataHDRDisplayPrimariesRedY,
+                                            mlt_properties_get_double(consumer_properties,
+                                                                      "hdr_red_y"));
+                        frameMeta->SetFloat(bmdDeckLinkFrameMetadataHDRDisplayPrimariesGreenX,
+                                            mlt_properties_get_double(consumer_properties,
+                                                                      "hdr_green_x"));
+                        frameMeta->SetFloat(bmdDeckLinkFrameMetadataHDRDisplayPrimariesGreenY,
+                                            mlt_properties_get_double(consumer_properties,
+                                                                      "hdr.green_y"));
+                        frameMeta->SetFloat(bmdDeckLinkFrameMetadataHDRDisplayPrimariesBlueX,
+                                            mlt_properties_get_double(consumer_properties,
+                                                                      "hdr_blue_x"));
+                        frameMeta->SetFloat(bmdDeckLinkFrameMetadataHDRDisplayPrimariesBlueY,
+                                            mlt_properties_get_double(consumer_properties,
+                                                                      "hdr_blue_y"));
+                        frameMeta->SetFloat(bmdDeckLinkFrameMetadataHDRWhitePointX,
+                                            mlt_properties_get_double(consumer_properties,
+                                                                      "hdr_white_x"));
+                        frameMeta->SetFloat(bmdDeckLinkFrameMetadataHDRWhitePointY,
+                                            mlt_properties_get_double(consumer_properties,
+                                                                      "hdr_white_y"));
+                        frameMeta->SetFloat(bmdDeckLinkFrameMetadataHDRMaxDisplayMasteringLuminance,
+                                            mlt_properties_get_double(consumer_properties,
+                                                                      "hdr_max_luminance"));
+                        frameMeta->SetFloat(bmdDeckLinkFrameMetadataHDRMinDisplayMasteringLuminance,
+                                            mlt_properties_get_double(consumer_properties,
+                                                                      "hdr_min_luminance"));
+                        frameMeta->SetFloat(bmdDeckLinkFrameMetadataHDRMaximumContentLightLevel,
+                                            mlt_properties_get_double(consumer_properties,
+                                                                      "hdr_max_cll"));
+                        frameMeta->SetFloat(bmdDeckLinkFrameMetadataHDRMaximumFrameAverageLightLevel,
+                                            mlt_properties_get_double(consumer_properties,
+                                                                      "hdr_max_fall"));
+                    }
+                }
+            }
+
             hr = m_deckLinkOutput->ScheduleVideoFrame(decklinkFrame,
                                                       m_count * m_duration,
                                                       m_duration,
@@ -739,25 +836,16 @@ protected:
                               frequency);
 
                 if (m_inChannels != m_outChannels) {
-                    int s = 0;
-                    int c = 0;
                     int size = mlt_audio_format_size(format, samples, m_outChannels);
                     int16_t *src = pcm;
-                    int16_t *dst = (int16_t *) mlt_pool_alloc(size);
-                    outBuff = dst;
-                    for (s = 0; s < samples; s++) {
-                        for (c = 0; c < m_outChannels; c++) {
-                            if (c < m_inChannels) {
-                                *dst = *src;
-                                src++;
-                            } else {
-                                // Fill silence if there are more out channels than in channels.
-                                *dst = 0;
-                            }
-                        }
-                        for (c = 0; c < m_inChannels - m_outChannels; c++) {
-                            // Drop samples if there are more in channels than out channels.
-                            src++;
+                    int16_t *dst = outBuff = (int16_t *) mlt_pool_alloc(size);
+                    for (int s = 0; s < samples; s++) {
+                        for (int c = 0; c < m_outChannels; c++) {
+                            int cOut = m_fix5p1 ? c == 2 ? 3 : c == 3 ? 2 : c : c;
+                            // Fill silence if there are more out channels than in channels.
+                            dst[s * m_outChannels + cOut] = (c < m_inChannels)
+                                                                ? src[s * m_inChannels + c]
+                                                                : 0;
                         }
                     }
                     pcm = outBuff;
@@ -776,7 +864,7 @@ protected:
                                                             samples,
                                                             streamTime,
                                                             frequency,
-                                                            (unsigned long *) &written);
+                                                            (unsigned int *) &written);
 #else
                 hr = m_deckLinkOutput->ScheduleAudioSamples(pcm,
                                                             samples,
@@ -844,12 +932,10 @@ protected:
 
         // step forward frames counter if underrun
         if (bmdOutputFrameDisplayedLate == completed) {
-            mlt_log_verbose(getConsumer(),
-                            "ScheduledFrameCompleted: bmdOutputFrameDisplayedLate == completed\n");
+            mlt_log_debug(getConsumer(), "ScheduledFrameCompleted: bmdOutputFrameDisplayedLate\n");
         }
         if (bmdOutputFrameDropped == completed) {
-            mlt_log_verbose(getConsumer(),
-                            "ScheduledFrameCompleted: bmdOutputFrameDropped == completed\n");
+            mlt_log_debug(getConsumer(), "ScheduledFrameCompleted: bmdOutputFrameDropped\n");
             m_count++;
             ScheduleNextFrame(false);
         }
