@@ -3,8 +3,8 @@
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  */
-
 #include "base.hpp"
+#include "model/assets/assets.hpp"
 
 QString glaxnimate::io::ImportExport::name_filter() const
 {
@@ -23,7 +23,7 @@ QString glaxnimate::io::ImportExport::name_filter() const
 }
 
 
-QByteArray glaxnimate::io::ImportExport::save(model::Document* document, const QVariantMap& setting_values, const QString& filename)
+QByteArray glaxnimate::io::ImportExport::save(model::Composition* comp, const QVariantMap& setting_values, const QString& filename)
 {
     QByteArray data;
     QBuffer file(&data);
@@ -31,18 +31,24 @@ QByteArray glaxnimate::io::ImportExport::save(model::Document* document, const Q
 
     QVariantMap clean_setting_values = setting_values;
 
-    if ( auto settings = save_settings(document) )
+    if ( auto settings = save_settings(comp) )
     {
         for ( const auto& setting : *settings )
             clean_setting_values[setting.slug] = setting.get_variant(clean_setting_values);
     }
 
-    if ( !save(file, filename, document, clean_setting_values) )
+    if ( !save(file, filename, comp, clean_setting_values) )
         return {};
 
     return data;
 }
 
+bool glaxnimate::io::ImportExport::save ( QIODevice& file, const QString& filename, model::Document* document, const QVariantMap& setting_values )
+{
+    if ( document->assets()->compositions->values.empty() )
+        return false;
+    return save(file, filename, document->assets()->compositions->values[0], setting_values);
+}
 
 bool glaxnimate::io::ImportExport::open(QIODevice& file, const QString& filename,
                     model::Document* document, const QVariantMap& setting_values)
@@ -57,13 +63,13 @@ bool glaxnimate::io::ImportExport::open(QIODevice& file, const QString& filename
 }
 
 bool glaxnimate::io::ImportExport::save(QIODevice& file, const QString& filename,
-                    model::Document* document, const QVariantMap& setting_values)
+                    model::Composition* comp, const QVariantMap& setting_values)
 {
     if ( !file.isOpen() && auto_open() )
         if ( !file.open(QIODevice::WriteOnly) )
             return false;
 
-    bool ok = on_save(file, filename, document, setting_values);
+    bool ok = on_save(file, filename, comp, setting_values);
     emit completed(ok);
     return ok;
 }
@@ -79,7 +85,7 @@ bool glaxnimate::io::ImportExport::load(model::Document* document, const QByteAr
 
     QVariantMap clean_setting_values = setting_values;
 
-    if ( auto settings = save_settings(document) )
+    if ( auto settings = open_settings() )
     {
         for ( const auto& setting : *settings )
             clean_setting_values[setting.slug] = setting.get_variant(clean_setting_values);
