@@ -30,21 +30,19 @@ bool io::glaxnimate::GlaxnimateFormat::on_open ( QIODevice& file, const QString&
 
     QJsonObject top_level = jdoc.object();
 
-    if ( !top_level["animation"].isObject() )
-    {
-        error(tr("Missing animation object"));
-        return false;
-    }
-
     int document_version = top_level["format"].toObject()["format_version"].toInt(0);
-    detail::ImportState state(this, document, document_version);
     if ( document_version > format_version )
         warning(tr("Opening a file from a newer version of Glaxnimate"));
 
-    state.load_metadata(document, top_level);
-    state.load_object(document->assets(), top_level[document_version < 3 ? "defs" : "assets"].toObject());
-    state.load_object(document->main(), top_level["animation"].toObject());
-    state.resolve();
+    detail::ImportState state(this, document, document_version);
+    state.load_document(top_level);
+
+    if ( document->assets()->compositions->values.empty() )
+    {
+        document->assets()->compositions->values.insert(std::make_unique<model::Composition>(document));
+        error(tr("Missing composition"));
+        return false;
+    }
 
     return true;
 }
