@@ -1,3 +1,9 @@
+/*
+ * SPDX-FileCopyrightText: 2019-2023 Mattia Basaglia <dev@dragon.best>
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
+ */
+
 #pragma once
 
 #include <set>
@@ -9,8 +15,7 @@
 
 #include "lottie_private_common.hpp"
 #include "io/svg/svg_parser.hpp"
-
-#include <QDebug>
+#include "model/animation/join_animatables.hpp"
 
 namespace glaxnimate::io::lottie::detail {
 
@@ -34,10 +39,11 @@ public:
     {
         load_version(json);
         load_meta(json["meta"]);
-        load_animation_container(json, document->main()->animation.get());
-        load_assets(json["assets"].toArray());
+        main = document->assets()->compositions->values.insert(std::make_unique<model::Composition>(document));
+        auto comps = load_assets(json["assets"].toArray());
         load_fonts(json["fonts"]["list"].toArray());
-        load_composition(json, document->main());
+        load_composition(json, main);
+        load_comps(comps);
     }
 
 private:
@@ -100,6 +106,23 @@ private:
         layer_indices.clear();
         deferred.clear();
 
+        if ( composition != main )
+        {
+            composition->width.set(main->width.get());
+            composition->height.set(main->height.get());
+            composition->fps.set(main->fps.get());
+            composition->animation->first_frame.set(main->animation->first_frame.get());
+            composition->animation->last_frame.set(main->animation->last_frame.get());
+        }
+
+        if ( json.contains("fr") )
+            composition->fps.set(json["fr"].toDouble());
+        if ( json.contains("w") )
+            composition->width.set(json["w"].toInt());
+        if ( json.contains("h") )
+            composition->height.set(json["h"].toInt());
+
+        load_animation_container(json, composition->animation.get());
         load_basic(json, composition);
 
         {
@@ -145,12 +168,13 @@ private:
 
         std::unique_ptr<model::ShapeElement> inner_shape;
         bool start_mask = json["td"].toInt();
+        start_mask = false;
 
         if ( ty == 0 )
         {
             inner_shape = load_precomp_layer(json);
 
-            auto op = document->main()->animation->last_frame.get();
+            auto op = this->composition->animation->last_frame.get();
             if ( json.contains("parent") || referenced.count(index) || json["ip"].toDouble() != 0 ||
                 json["op"].toDouble(op) != op || start_mask
             )
@@ -299,7 +323,16 @@ private:
             }
         }
 
-        load_animation_container(json, layer->animation.get());
+        if ( !json.contains("ip") && !json.contains("op") )
+        {
+            auto comp = layer->owner_composition();
+            layer->animation->first_frame.set(comp->animation->first_frame.get());
+            layer->animation->last_frame.set(comp->animation->last_frame.get());
+        }
+        else
+        {
+            load_animation_container(json, layer->animation.get());
+        }
 
         if ( !layer->shapes.empty() )
             return;
@@ -329,6 +362,8 @@ private:
                 auto shape_target = std::make_unique<model::Layer>(document);
                 target = shape_target.get();
                 shape_target->name.set(layer->name.get());
+                shape_target->animation->first_frame.set(layer->animation->first_frame.get());
+                shape_target->animation->last_frame.set(layer->animation->last_frame.get());
                 layer->shapes.insert(std::move(shape_target));
 
                 document->set_best_name(clip, QObject::tr("Clip"));
@@ -503,6 +538,29 @@ private:
         load_basic(transform, tf);
         if ( transform.contains("o") && opacity )
             load_animated(opacity, transform["o"], FloatMult(100));
+
+        if ( transform.contains("p") )
+        {
+            auto pos = transform["p"].toObject();
+            if ( pos.contains("x") && pos.contains("y") )
+            {
+                model::Document dummydoc("");
+                model::Object dummy(&dummydoc);
+                model::AnimatedProperty<float> px(&dummy, "", 0);
+                model::AnimatedProperty<float> py(&dummy, "", 0);
+                load_animated(&px, pos["x"], {});
+                load_animated(&py, pos["y"], {});
+
+                model::JoinAnimatables join({&px, &py});
+                join.apply_to(&tf->position, [](float x, float y) -> QPointF {
+                    return QPointF(x, y);
+                }, &px, &py);
+            }
+            else
+            {
+                load_animated(&tf->position, transform["p"], {});
+            }
+        }
     }
 
     void load_styler(model::Styler* styler, const QJsonObject& json_obj)
@@ -529,8 +587,28 @@ private:
             gradient->colors.set(colors);
             load_properties(gradient, fields["Gradient"], json_obj, props);
 
-            /// \todo load highlight from h/a if present
-            gradient->highlight.set(gradient->start_point.get());
+            if ( json_obj.contains("h") || json_obj.contains("a") )
+            {
+                model::Document dummydoc("");
+                model::Object dummy(&dummydoc);
+                model::AnimatedProperty<float> length(&dummy, "", 0);
+                model::AnimatedProperty<float> angle(&dummy, "", 0);
+                if ( json_obj.contains("h") )
+                    load_animated(&length, json_obj["h"], {});
+                if ( json_obj.contains("a") )
+                    load_animated(&angle, json_obj["a"], {});
+
+                glaxnimate::model::JoinAnimatables join({&gradient->start_point, &gradient->end_point, &length, &angle});
+                join.apply_to(&gradient->highlight, [](const QPointF& p, const QPointF& e, float length, float angle) -> QPointF {
+                    angle = math::deg2rad(angle + 90);
+                    length = math::length(e - p) * length / 100;
+                    return p + math::from_polar<QPointF>(length, angle);
+                }, &gradient->start_point, &gradient->end_point, &length, &angle);
+            }
+            else
+            {
+                gradient->highlight.set(gradient->start_point.get());
+            }
 
             auto jcolors = json_obj["g"].toObject();
             load_animated(&colors->colors, jcolors["k"], GradientLoad{jcolors["p"].toInt()});
@@ -607,7 +685,7 @@ private:
         for ( const FieldInfo& field : fields )
         {
             avail_obj_keys.erase(field.lottie);
-            if ( field.mode != Auto || !json_obj.contains(field.lottie) )
+            if ( field.mode >= Ignored || !json_obj.contains(field.lottie) )
                 continue;
 
             model::BaseProperty * prop = obj->get_property(field.name);
@@ -621,6 +699,10 @@ private:
             {
                 load_animated(static_cast<model::AnimatableBase*>(prop), json_obj[field.lottie], field.transform);
             }
+            else if ( field.mode == AnimatedToStatic )
+            {
+                load_static(prop, json_obj[field.lottie], field.transform);
+            }
             else
             {
                 load_value(prop, json_obj[field.lottie], field.transform);
@@ -793,6 +875,21 @@ private:
             emit format->warning(QObject::tr("Invalid value for %1").arg(prop->name()));
     }
 
+    void load_static(model::BaseProperty * prop, const QJsonValue& val, const TransformFunc& trans)
+    {
+        if ( val.isObject() )
+        {
+            QJsonObject obj = val.toObject();
+            if ( obj.contains("k") )
+            {
+                load_value(prop, obj["k"], trans);
+                return;
+            }
+        }
+
+        load_value(prop, val, trans);
+    }
+
     void load_animated(model::AnimatableBase* prop, const QJsonValue& val, const TransformFunc& trans)
     {
         if ( !val.isObject() )
@@ -900,9 +997,9 @@ private:
         return {keyframe_bezier_handle_comp(val["x"]), keyframe_bezier_handle_comp(val["y"])};
     }
 
-    void load_assets(const QJsonArray& assets)
+    std::vector<std::pair<QJsonObject, model::Composition*>> load_assets(const QJsonArray& assets)
     {
-        std::vector<std::pair<QJsonObject, model::Precomposition*>> comps;
+        std::vector<std::pair<QJsonObject, model::Composition*>> comps;
 
         for ( const auto& assetv : assets )
         {
@@ -913,6 +1010,11 @@ private:
                 comps.emplace_back(asset, load_asset_precomp(asset));
         }
 
+        return comps;
+    }
+
+    void load_comps(const std::vector<std::pair<QJsonObject, model::Composition*>>& comps)
+    {
         for ( const auto& p : comps )
             load_composition(p.first, p.second);
     }
@@ -926,24 +1028,36 @@ private:
             format->warning(io::lottie::LottieFormat::tr("Duplicate Bitmap ID: %1").arg(id));
         bitmap_ids[id] = bmp;
 
+        if ( asset.contains("nm") )
+            bmp->name.set(asset["nm"].toString());
+
         if ( asset["e"].toInt() )
         {
             bmp->from_url(asset["p"].toString());
         }
         else
         {
-            QDir dir(asset["u"].toString());
-            bmp->from_file(dir.filePath(asset["p"].toString()));
+            QString path = asset["u"].toString();
+            if ( path.contains("://") )
+            {
+                path += asset["p"].toString();
+                bmp->from_url(path);
+            }
+            else
+            {
+                QDir dir(path);
+                bmp->from_file(dir.filePath(asset["p"].toString()));
+            }
         }
     }
 
-    model::Precomposition* load_asset_precomp(QJsonObject asset)
+    model::Composition* load_asset_precomp(QJsonObject asset)
     {
-        auto comp = document->assets()->precompositions->values.insert(std::make_unique<model::Precomposition>(document));
+        auto comp = document->assets()->compositions->values.insert(std::make_unique<model::Composition>(document));
 
         QString id = asset["id"].toString();
         if ( precomp_ids.count(id) )
-            format->warning(io::lottie::LottieFormat::tr("Duplicate Precomposition ID: %1").arg(id));
+            format->warning(io::lottie::LottieFormat::tr("Duplicate Composition ID: %1").arg(id));
         precomp_ids[id] = comp;
 
         comp->name.set(id);
@@ -1068,12 +1182,13 @@ private:
     model::Composition* composition = nullptr;
     app::log::Log logger{"Lottie Import"};
     QMap<QString, model::Bitmap*> bitmap_ids;
-    QMap<QString, model::Precomposition*> precomp_ids;
+    QMap<QString, model::Composition*> precomp_ids;
     QMap<QString, FontInfo> fonts;
     model::Layer* mask = nullptr;
     model::DocumentNode* current_node = nullptr;
     model::Layer* current_layer = nullptr;
     std::array<int, 3> version = {5,5,1};
+    model::Composition* main = nullptr;
 };
 
 
