@@ -38,6 +38,34 @@ public:
     {
         QDomElement element;
 
+        bool apply_motion(model::AnimatedProperty<QPointF>& prop, const QPointF& delta_pos = {}, model::Property<bool>* auto_orient = nullptr) const
+        {
+            auto motion = properties.find("motion");
+            if ( motion == properties.end() )
+                return false;
+
+            if ( auto_orient )
+                auto_orient->set(motion->second.auto_orient);
+
+            for ( const auto& kf : motion->second.keyframes )
+                prop.set_keyframe(kf.time, QPointF())->set_transition(kf.transition);
+
+            if ( qFuzzyIsNull(math::length(delta_pos)) )
+            {
+                prop.set_bezier(motion->second.motion);
+            }
+            else
+            {
+                math::bezier::Bezier bez = motion->second.motion;
+                for ( auto& p : bez )
+                    p.translate(delta_pos);
+
+                prop.set_bezier(bez);
+            }
+
+            return true;
+        }
+
         bool prepare_joined(std::vector<JoinedProperty>& props) const override
         {
             for ( auto& p : props )
@@ -228,7 +256,24 @@ public:
         return values;
     }
 
-    void parse_animate(const QDomElement& animate, AnimatedProperty& prop)
+    void register_time_range(model::FrameTime start_time, model::FrameTime end_time)
+    {
+        if ( !kf_range_initialized )
+        {
+            kf_range_initialized = true;
+            min_kf = start_time;
+            max_kf = end_time;
+        }
+        else
+        {
+            if ( min_kf > start_time )
+                min_kf = start_time;
+            if ( max_kf < end_time )
+                max_kf = end_time;
+        }
+    }
+
+    void parse_animate(const QDomElement& animate, AnimatedProperty& prop, bool motion)
     {
         if ( !prop.keyframes.empty() )
         {
@@ -253,7 +298,56 @@ public:
             return;
         }
 
-        std::vector<ValueVariant> values = get_values(animate);
+        register_time_range(start_time, end_time);
+
+        std::vector<ValueVariant> values;
+
+        if ( motion )
+        {
+            if ( !animate.hasAttribute("path") )
+            {
+                warning("Missing path for animateMotion");
+                return;
+            }
+
+            if ( animate.hasAttribute("rotate") )
+            {
+                QString rotate = animate.attribute("rotate");
+                if ( rotate == "auto" )
+                {
+                    prop.auto_orient = true;
+                }
+                else
+                {
+                    bool is_number = false;
+                    int degrees = rotate.toInt(&is_number) % 360;
+                    if ( !is_number || degrees != 0 )
+                    {
+                        warning("The only supported values for animateMotion.rotate are auto or 0");
+                        prop.auto_orient = rotate == "auto-reverse";
+                    }
+                }
+            }
+
+            auto mbez = PathDParser(animate.attribute("path")).parse();
+
+            /// \todo Automatically add Hold transitions for sub-bezier end points
+            for ( const auto& b : mbez.beziers() )
+            {
+                for ( const auto& p : b )
+                {
+                    values.push_back(std::vector<qreal>{p.pos.x(), p.pos.y()});
+                    prop.motion.push_back(p);
+                }
+            }
+
+            /// \todo keyPoints
+        }
+        else
+        {
+            values = get_values(animate);
+        }
+
         if ( values.empty() )
             return;
 
@@ -271,7 +365,7 @@ public:
             );
             if ( strings.size() != int(values.size()) )
             {
-                warning("`keyTimes` and `values` mismatch");
+                warning(QString("`keyTimes` (%1) and `values` (%2) mismatch").arg(strings.size()).arg(int(values.size())));
                 return;
             }
 
@@ -334,47 +428,52 @@ public:
             prop.keyframes.push_back({times[i], values[i], transitions[i]});
     }
 
-    AnimatedProperties parse_animated_properties(const QDomElement& parent)
+    void store_animate(const QString& target, const QDomElement& animate)
+    {
+        stored_animate[target].push_back(animate);
+    }
+
+    template<class FuncT>
+    AnimatedProperties parse_animated_elements(const QDomElement& parent, const FuncT& func)
     {
         AnimatedProperties props;
         props.element = parent;
 
-        for ( const auto& domnode : ItemCountRange(parent.childNodes()) )
-        {
-            if ( domnode.isElement() )
-            {
-                auto child = domnode.toElement();
+        for ( const auto& child: ElementRange(parent) )
+            func(child, props);
 
-                if ( child.tagName() == "animate" )
-                {
-                    if ( child.hasAttribute("attributeName") )
-                        parse_animate(child, props.properties[child.attribute("attributeName")]);
-                }
+        if ( parent.hasAttribute("id") )
+        {
+            auto it = stored_animate.find(parent.attribute("id"));
+            if ( it != stored_animate.end() )
+            {
+                for ( const auto& child: it->second )
+                    func(child, props);
             }
         }
 
         return props;
     }
 
+
+    AnimatedProperties parse_animated_properties(const QDomElement& parent)
+    {
+        return parse_animated_elements(parent, [this](const QDomElement& child, AnimatedProperties& props){
+            if ( child.tagName() == "animate" && child.hasAttribute("attributeName") )
+                parse_animate(child, props.properties[child.attribute("attributeName")], false);
+            else if ( child.tagName() == "animateMotion" )
+                parse_animate(child, props.properties["motion"], true);
+        });
+    }
+
     AnimatedProperties parse_animated_transform(const QDomElement& parent)
     {
-        AnimatedProperties props;
-        props.element = parent;
-
-        for ( const auto& domnode : ItemCountRange(parent.childNodes()) )
-        {
-            if ( domnode.isElement() )
-            {
-                auto child = domnode.toElement();
-                if ( child.tagName() == "animateTransform" )
-                {
-                    if ( child.hasAttribute("type") && child.attribute("attributeName") == "transform" )
-                        parse_animate(child, props.properties[child.attribute("type")]);
-                }
-            }
-        }
-
-        return props;
+        return parse_animated_elements(parent, [this](const QDomElement& child, AnimatedProperties& props){
+            if ( child.tagName() == "animateTransform" && child.hasAttribute("type") && child.attribute("attributeName") == "transform" )
+                parse_animate(child, props.properties[child.attribute("type")], false);
+            else if ( child.tagName() == "animateMotion" )
+                parse_animate(child, props.properties["motion"], true);
+        });
     }
 
     void warning(const QString& msg)
@@ -384,7 +483,11 @@ public:
     }
 
     qreal fps = 60;
+    qreal min_kf = 0;
+    qreal max_kf = 0;
+    bool kf_range_initialized = false;
     std::function<void(const QString&)> on_warning;
+    std::unordered_map<QString, std::vector<QDomElement>> stored_animate;
     static const QRegularExpression separator;
     static const QRegularExpression clock_re;
     static const QRegularExpression frame_separator_re;
