@@ -713,7 +713,9 @@ mlt_transition transition_affine_init(mlt_profile profile,
     mlt_transition transition = mlt_transition_new();
     if (transition != NULL) {
         mlt_properties_set_int(MLT_TRANSITION_PROPERTIES(transition), "distort", 0);
-        mlt_properties_set(MLT_TRANSITION_PROPERTIES(transition), "rect", "0%/0%:100%x100%:100%");
+        mlt_properties_set(MLT_TRANSITION_PROPERTIES(transition),
+                           "rect",
+                           arg ? arg : "0%/0%:100%x100%:100%");
         // Inform apps and framework that this is a video only transition
         mlt_properties_set_int(MLT_TRANSITION_PROPERTIES(transition), "_transition_type", 1);
         mlt_properties_set_int(MLT_TRANSITION_PROPERTIES(transition), "fill", 1);
