@@ -1,6 +1,6 @@
 /*
- * filter_lightshow.cpp -- animate color to the audio
- * Copyright (C) 2015 Meltytech, LLC
+ * filter_qtblend.cpp -- Qt composite filter
+ * Copyright (C) 2015-2025 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,8 @@
 #include <QPainter>
 #include <QTransform>
 
+#define MLT_QTBLEND_MAX_DIMENSION (16000)
+
 /** Get the image.
 */
 static int filter_get_image(mlt_frame frame,
@@ -59,26 +61,25 @@ static int filter_get_image(mlt_frame frame,
     double consumer_ar = mlt_profile_sar(profile);
 
     // Destination rect
-    mlt_rect rect = {0,
-                     0,
-                     normalized_width * mlt_profile_scale_width(profile, *width),
-                     normalized_height * mlt_profile_scale_height(profile, *height),
-                     1.0};
+    mlt_rect rect = {0, 0, (double) normalized_width, (double) normalized_height, 1.0};
     int b_width = mlt_properties_get_int(frame_properties, "meta.media.width");
     int b_height = mlt_properties_get_int(frame_properties, "meta.media.height");
+    bool distort = mlt_properties_get_int(properties, "distort");
+
     if (b_height == 0) {
         b_width = normalized_width;
         b_height = normalized_height;
     }
     // Special case - aspect_ratio = 0
     if (mlt_frame_get_aspect_ratio(frame) == 0) {
-        double output_ar = mlt_profile_sar(profile);
-        mlt_frame_set_aspect_ratio(frame, output_ar);
+        mlt_frame_set_aspect_ratio(frame, consumer_ar);
     }
     double b_ar = mlt_frame_get_aspect_ratio(frame);
     double b_dar = b_ar * b_width / b_height;
     double opacity = 1.0;
 
+    // If the _qtblend_scaled property is defined, a qtblend filter was already applied
+    double qtblendScaleX = qMin(1., mlt_properties_get_double(frame_properties, "qtblend_scalingx"));
     if (mlt_properties_get(properties, "rect")) {
         rect = mlt_properties_anim_get_rect(properties, "rect", position, length);
         if (::strchr(mlt_properties_get(properties, "rect"), '%')) {
@@ -87,37 +88,80 @@ static int filter_get_image(mlt_frame frame,
             rect.w *= normalized_width;
             rect.h *= normalized_height;
         }
-        double scale = mlt_profile_scale_width(profile, *width);
+    }
+    if (qtblendScaleX > 0.) {
+        // Another qtblend filter was already applied
+        // In this case, the *width and *height are set to the source resolution to ensure we don't lose too much details on multiple scaling operations
+        // We requested a image with full media resolution, adjust rect to profile
+        // Check if we have consumer scaling enabled since we cannot use *width and *height
+        double qtblendScaleY = qMin(1.,
+                                    mlt_properties_get_double(frame_properties, "qtblend_scalingy"));
+        // Consumer scaling was already applied to b_width/b_height
+        // Always request an image that follows the consumer aspect ratio
+        double consumer_dar = normalized_width * consumer_ar / normalized_height;
+        int tmpWidth = b_width;
+        int tmpHeight = b_height;
+        double scaleFactor = qMax(*width / rect.w, *height / rect.h);
+        if (scaleFactor > 1.) {
+            // Use the highest necessary resolution image
+            tmpWidth *= scaleFactor;
+            tmpHeight *= scaleFactor;
+        }
+        if (consumer_dar > b_dar) {
+            *width = qBound(qRound(normalized_width * qtblendScaleX),
+                            tmpWidth,
+                            MLT_QTBLEND_MAX_DIMENSION);
+            *height = qRound(*width * consumer_ar * normalized_height / normalized_width);
+        } else {
+            *height = qBound(qRound(normalized_height * qtblendScaleY),
+                             tmpHeight,
+                             MLT_QTBLEND_MAX_DIMENSION);
+            *width = qRound(*height * normalized_width / normalized_height / consumer_ar);
+        }
+        // Adjust rect to new scaling
+        double scale = (double) *width / normalized_width;
         if (scale != 1.0) {
             rect.x *= scale;
             rect.w *= scale;
         }
-        scale = mlt_profile_scale_height(profile, *height);
+        scale = (double) *height / normalized_height;
         if (scale != 1.0) {
             rect.y *= scale;
             rect.h *= scale;
         }
-        transform.translate(rect.x, rect.y);
-        opacity = rect.o;
-        hasAlpha = rect.o < 1 || rect.x != 0 || rect.y != 0 || rect.w != *width
-                   || rect.h != *height;
-
-        if (mlt_properties_get_int(properties, "distort") == 0) {
-            b_height = qMax(1, qMin(qRound(rect.h), b_height));
-            b_width = qMax(1, qRound(b_height * b_dar / b_ar / consumer_ar));
-        } else {
-            b_width = qMax(1, qRound(b_width * b_ar / consumer_ar));
-        }
-        if (!hasAlpha && (b_width < *width || b_height < *height)) {
-            hasAlpha = true;
-        }
     } else {
-        b_width = *width;
-        b_height = *height;
-        if (b_width < normalized_width || b_height < normalized_height) {
-            hasAlpha = true;
+        // First instance of a qtblend filter
+        // Check if requested frame size is scaled
+        double scalex = mlt_profile_scale_width(profile, *width);
+        double scaley = mlt_profile_scale_height(profile, *height);
+
+        // Store consumer scaling for further uses
+        mlt_properties_set_double(frame_properties, "qtblend_scalingx", scalex);
+        mlt_properties_set_double(frame_properties, "qtblend_scalingy", scaley);
+        // Apply scaling
+        if (scalex != 1.0) {
+            rect.x *= scalex;
+            rect.w *= scalex;
+            if (b_width < normalized_width) {
+                // Adjust scale so that we don't request too small images
+                scalex = qBound(scalex, normalized_width * scalex / b_width, 1.);
+            }
+            // Apply consumer scaling to the source image
+            if (scalex < 1.) {
+                b_width *= scalex;
+                b_height *= scalex;
+            }
+        }
+        if (scaley != 1.0) {
+            rect.y *= scaley;
+            rect.h *= scaley;
         }
     }
+    transform.translate(rect.x, rect.y);
+    opacity = rect.o;
+    hasAlpha = rect.o < 1 || rect.x != 0 || rect.y != 0 || rect.w != *width || rect.h != *height
+               || rect.w / b_dar < *height || rect.h * b_dar < *width || b_width < *width
+               || b_height < *height;
 
     if (mlt_properties_get(properties, "rotation")) {
         double angle = mlt_properties_anim_get_double(properties, "rotation", position, length);
@@ -154,7 +198,6 @@ static int filter_get_image(mlt_frame frame,
     // fetch image
     *format = mlt_image_rgba;
     uint8_t *src_image = NULL;
-
     error = mlt_frame_get_image(frame, &src_image, format, &b_width, &b_height, 0);
 
     // Put source buffer into QImage
@@ -164,13 +207,12 @@ static int filter_get_image(mlt_frame frame,
     int image_size = mlt_image_format_size(*format, *width, *height, NULL);
 
     // resize to rect
-    if (mlt_properties_get_int(properties, "distort")) {
+    if (distort) {
         transform.scale(rect.w / b_width, rect.h / b_height);
     } else {
-        // Determine scale with respect to aspect ratio.
-        double geometry_dar = rect.w * consumer_ar / rect.h;
         double scale;
-        if (b_dar > geometry_dar) {
+        double resize_dar = rect.w * consumer_ar / rect.h;
+        if (b_dar >= resize_dar) {
             scale = rect.w / b_width;
         } else {
             scale = rect.h / b_height * b_ar;
