@@ -17,21 +17,24 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#include <string.h>
-#include <limits.h>
 #include <framework/mlt.h>
+#include <limits.h>
+#include <string.h>
 
-extern mlt_filter filter_rbpitch_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
+extern mlt_filter filter_rbpitch_init(mlt_profile profile,
+                                      mlt_service_type type,
+                                      const char *id,
+                                      char *arg);
 
-static mlt_properties metadata( mlt_service_type type, const char *id, void *data )
+static mlt_properties metadata(mlt_service_type type, const char *id, void *data)
 {
-	char file[ PATH_MAX ];
-	snprintf( file, PATH_MAX, "%s/rubberband/%s", mlt_environment( "MLT_DATA" ), (char*) data );
-	return mlt_properties_parse_yaml( file );
+    char file[PATH_MAX];
+    snprintf(file, PATH_MAX, "%s/rubberband/%s", mlt_environment("MLT_DATA"), (char *) data);
+    return mlt_properties_parse_yaml(file);
 }
 
 MLT_REPOSITORY
 {
-	MLT_REGISTER( mlt_service_filter_type, "rbpitch", filter_rbpitch_init );
-	MLT_REGISTER_METADATA( mlt_service_filter_type, "rbpitch", metadata, "filter_rbpitch.yml" );
+    MLT_REGISTER(mlt_service_filter_type, "rbpitch", filter_rbpitch_init);
+    MLT_REGISTER_METADATA(mlt_service_filter_type, "rbpitch", metadata, "filter_rbpitch.yml");
 }
