@@ -3,23 +3,23 @@
 
 # Import required modules
 from __future__ import print_function
-import mlt
+import mlt7 as mlt
 
 # Start the mlt system
-mlt.Factory().init( )
+mlt.Factory().init()
 
 # Create the consumer
-c = mlt.Consumer( mlt.Profile(), "avformat" )
+c = mlt.Consumer(mlt.Profile(), "avformat")
 
 # Ask for video codecs supports
-c.set( 'vcodec', 'list' )
+c.set('vcodec', 'list')
 
 # Start the consumer to generate the list
 c.start()
 
 # Get the vcodec property
-codecs = mlt.Properties( c.get_data( 'vcodec' ) )
+codecs = mlt.Properties(c.get_data('vcodec'))
 
 # Print the list of codecs
-for i in range( 0, codecs.count()):
-	print(codecs.get( i ))
+for i in range(0, codecs.count()):
+	print(codecs.get(i))
